@@ -1,5 +1,5 @@
 /* Substitution of environment variables in shell format strings.
-   Copyright (C) 2003-2007, 2012, 2015-2016 Free Software Foundation, Inc.
+   Copyright (C) 2003-2007, 2012, 2018-2020 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <locale.h>
 
 #include "closeout.h"
@@ -34,6 +35,7 @@
 #include "basename.h"
 #include "xalloc.h"
 #include "propername.h"
+#include "binary-io.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -73,10 +75,8 @@ main (int argc, char *argv[])
   /* Set program name for message texts.  */
   set_program_name (argv[0]);
 
-#ifdef HAVE_SETLOCALE
   /* Set locale via LC_ALL.  */
   setlocale (LC_ALL, "");
-#endif
 
   /* Set the text message domain.  */
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
@@ -110,11 +110,11 @@ main (int argc, char *argv[])
       printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
       /* xgettext: no-wrap */
       printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
-License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
+License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "2003-2007");
+              "2003-2020", "https://gnu.org/licenses/gpl.html");
       printf (_("Written by %s.\n"), proper_name ("Bruno Haible"));
       exit (EXIT_SUCCESS);
     }
@@ -139,6 +139,11 @@ There is NO WARRANTY, to the extent permitted by law.\n\
         default:
           abort ();
         }
+
+      /* The result is most often used in shell `...` expressions.
+         Therefore, on native Windows, don't produce CR/LF newlines.  */
+      set_binary_mode (STDOUT_FILENO, O_BINARY);
+
       print_variables (argv[optind++]);
     }
   else
@@ -212,11 +217,16 @@ standard input are substituted.\n"));
 When --variables is used, standard input is ignored, and the output consists\n\
 of the environment variables that are referenced in SHELL-FORMAT, one per line.\n"));
       printf ("\n");
-      /* TRANSLATORS: The placeholder indicates the bug-reporting address
-         for this package.  Please add _another line_ saying
+      /* TRANSLATORS: The first placeholder is the web address of the Savannah
+         project of this package.  The second placeholder is the bug-reporting
+         email address for this package.  Please add _another line_ saying
          "Report translation bugs to <...>\n" with the address for translation
          bugs (typically your translation team's web or email address).  */
-      fputs (_("Report bugs to <bug-gnu-gettext@gnu.org>.\n"), stdout);
+      printf(_("\
+Report bugs in the bug tracker at <%s>\n\
+or by email to <%s>.\n"),
+             "https://savannah.gnu.org/projects/gettext",
+             "bug-gettext@gnu.org");
     }
 
   exit (status);
@@ -439,8 +449,8 @@ do_getc ()
   if (c == EOF)
     {
       if (ferror (stdin))
-        error (EXIT_FAILURE, errno, _("\
-error while reading \"%s\""), _("standard input"));
+        error (EXIT_FAILURE, errno,
+               _("error while reading \"%s\""), _("standard input"));
     }
 
   return c;
